#!/usr/bin/env python3
import configparser
import os
import shutil
import subprocess
import sys
import time
from datetime import datetime
from pathlib import Path

import requests

channel_list = []
downloading = {}

# Default Config Settings
config = configparser.ConfigParser()
streamlink_location = "streamlink"
download_location = f"{Path.home()}/Downloads/Streams"
skip_ads = False


def load_config():
    print("Reading config file...")
    if os.path.exists("config.ini"):
        config.read("config.ini")
        if (
            config.has_option("settings", "streamlink_location")
            and not config["settings"]["streamlink_location"].strip()
        ):
            streamlink_location = config["settings"]["streamlink_location"]
        if (
            config.has_option("settings", "download_location")
            and not config["settings"]["download_location"].strip()
        ):
            download_location = config["settings"]["download_location"]
        if (
            config.has_option("settings", "skip_ads")
            and not config["settings"]["skip_ads"]
        ):
            skip_ads = bool(config["settings"]["skip_ads"])
        print("Config file loaded")
    else:
        print("No config file found using default values!")


def write_log(channel):
    """Writes the latest stdout of a process to log.txt"""
    with open("log.txt", "a") as log:
        line = downloading[channel].stdout.readline()
        if line:
            log.write(line.decode())


def download_stream(channel):
    """Downloads a given channel name in its own subprocess"""
    addtional_parms = ""
    if skip_ads:
        addtional_parms = "--twitch-proxy-playlist=https://lb-eu.cdn-perfprod.com,https://lb-eu2.cdn-perfprod.com,https://lb-na.cdn-perfprod.com,https://lb-as.cdn-perfprod.com,https://as.luminous.dev --twitch-disable-ads"
    file_name = f"{channel}_{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}.ts"
    # cmd = [
    #    f"{streamlink_location}",
    #    "--loglevel none",
    #    "--retry-max 10",
    #    f"{addtional_parms}",
    #    f"-o {download_location}/{channel}/{file_name}",
    #    f"twitch.tv/{channel}",
    #    "best",
    # ]
    # print(cmd)
    downloading[channel] = subprocess.Popen(
        [
            f"{streamlink_location}",
            "--loglevel none",
            "--retry-max 10",
            f"{addtional_parms}",
            f"-o {download_location}/{channel}/{file_name}",
            "twitch.tv/ActionButton",
            "best",
        ]
    )
    # write_log(channel)


def check_system():
    """Makes sure everything is place for the script to run"""
    # Checks if streamlink is in the systems path
    if not shutil.which("streamlink"):
        sys.exit("ERROR: streamlink is not found in the systems path!")

    # TODO: Combine the channel_list into the config
    # Checks if the channel_list exists and if not makes one
    if not os.path.exists("channel_list.txt"):
        print("ERROR:'channel_list.txt' does not exist, creating now!")
        with open("channel_list.txt", "w"):
            pass  # Creates empty file
        sys.exit("Please populate the channel_list.txt with one channel per line!")

    # Make sure the download location exists
    if not os.path.exists(download_location):
        print(
            f"Download destination does not exist.\n Creating now at {download_location}"
        )
        os.makedirs(download_location)


def stop_downloads():
    """Goes through every process and stops it if running"""
    print("\nCleaning up...")
    for name, proc in downloading.items():
        proc.terminate()
        print(f"Stopping download of {name}")


def main():
    """Main entry point of the app"""
    # Grab all the channels from channel_list.txt and put them in a list
    with open("channel_list.txt", "r") as file:
        # channel_list = file.readlines()
        channel_list = [
            line for line in file if line.strip()
        ]  # Removes all white spaces per line

    # Run untill progam is killed
    while True:
        # Exits the program if there is no channels to grab
        if not channel_list:
            sys.exit("Please populate the channel_list.txt with one channel per line!")
        print("\n------------------------------------")
        for channel in channel_list:
            channel = channel.strip()
            # TODO: Have steamlink itself check if the channel is live
            contents = requests.get("https://www.twitch.tv/" + channel).content.decode(
                "utf-8"
            )
            if "isLiveBroadcast" in contents:
                print(f"\033[1m{channel}\033[0m is \033[32mlive\033[0m!")
                if channel not in downloading:
                    download_stream(channel)
                else:
                    print(f"{channel} is already downloading")
                    write_log(channel)
            else:
                print("\033[1m" + channel + "\033[0m is \033[31mnot live\033[0m.")
                if channel in downloading:
                    del downloading[channel]
                    print(f"{channel} is no longer downloading")
            time.sleep(1)  # Wait one second before going to next channel
        print(
            "\n\033[3mLast checked: " + datetime.now().strftime("%H:%M:%S") + "\033[0m"
        )
        print("------------------------------------")
        time.sleep(60)  # Wait 60 Seconds before trying again


if __name__ == "__main__":
    """ This is executed when run from the command line """
    try:
        load_config()
        check_system()
        main()
    finally:
        stop_downloads()
